@@ -11,6 +11,7 @@
 
 #include <stdio.h>
 #include <math.h>
+#include <string.h>
 
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmarshal.h>
@@ -122,6 +123,48 @@ sync_zoom_level_items(GgvControl *control)
 								 "state", "1", NULL);
 }
 
+static void
+sync_auto_fit_items(GgvControl *control)
+{
+	int i;
+	Bonobo_UIContainer container;
+	GgvPostScriptView *ps_view = control->priv->ps_view;
+	GtkWidget *gs;
+
+	container = bonobo_ui_component_get_container (control->priv->uic);
+	if (container == CORBA_OBJECT_NIL)
+		return;
+
+	gs = ggv_postscript_view_get_widget(ps_view);
+
+	switch(gtk_gs_get_zoom_mode(GTK_GS(gs))) {
+	case GTK_GS_ZOOM_ABSOLUTE:
+		bonobo_ui_component_set_prop(control->priv->uic,
+									 "/commands/AutoFitNone",
+									 "state", "1", NULL);
+		break;
+	case GTK_GS_ZOOM_FIT_WIDTH:
+		bonobo_ui_component_set_prop(control->priv->uic,
+									 "/commands/AutoFitWidth",
+									 "state", "1", NULL);
+		break;
+	case GTK_GS_ZOOM_FIT_PAGE:
+		bonobo_ui_component_set_prop(control->priv->uic,
+									 "/commands/AutoFitPage",
+									 "state", "1", NULL);
+		break;
+	default:
+		break;
+	}
+}
+
+static void
+sync_command_state(GgvControl *control)
+{
+	sync_auto_fit_items(control);
+	sync_zoom_level_items(control);
+}
+
 static void
 zoomable_set_frame_cb (BonoboZoomable *zoomable, GgvControl *control)
 {
@@ -147,8 +190,9 @@ zoomable_set_zoom_level_cb (BonoboZoomable *zoomable, float new_zoom_level,
 		(control->priv->ps_view);
 
 	if(control->priv->zoom_adj) {
-		gtk_adjustment_set_value(GTK_ADJUSTMENT(control->priv->zoom_adj),
-								 new_zoom_level*100.0);
+		if(fabs(GTK_ADJUSTMENT(control->priv->zoom_adj)->value - new_zoom_level*100.0) > 0.01)
+			gtk_adjustment_set_value(GTK_ADJUSTMENT(control->priv->zoom_adj),
+									 new_zoom_level*100.0);
 	}
 
 	bonobo_zoomable_report_zoom_level_changed
@@ -278,6 +322,37 @@ listener_ZoomLevel_cb(BonoboUIComponent *uic, const char *path,
 	}
 }
 
+static void
+listener_AutoFitMode_cb(BonoboUIComponent *uic, const char *path,
+						Bonobo_UIComponent_EventType type, const char *state,
+						gpointer user_data)
+{
+	GgvControl *control;
+	GtkGS *gs;
+	gint i;
+
+	g_return_if_fail(user_data != NULL);
+	g_return_if_fail(GGV_IS_CONTROL(user_data));
+
+	if(type != Bonobo_UIComponent_STATE_CHANGED)
+		return;
+
+	if(!state || !atoi(state))
+		return;
+
+	control = GGV_CONTROL(user_data);
+	gs = GTK_GS(ggv_postscript_view_get_widget(control->priv->ps_view));
+
+	if(!strcmp(path, "AutoFitNone"))
+		gtk_gs_set_zoom_mode(gs, GTK_GS_ZOOM_ABSOLUTE);
+	if(!strcmp(path, "AutoFitWidth"))
+		gtk_gs_set_zoom_mode(gs, GTK_GS_ZOOM_FIT_WIDTH);
+	if(!strcmp(path, "AutoFitPage"))
+		gtk_gs_set_zoom_mode(gs, GTK_GS_ZOOM_FIT_PAGE);
+
+	gtk_widget_unref(GTK_WIDGET(gs));
+}
+
 static gboolean
 ggv_control_button_press_event(GtkWidget *widget, GdkEventButton *event,
 							   gpointer data)
@@ -300,7 +375,7 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 	GtkGSOrientation orientation = gtk_gs_get_orientation(gs);
 	GgvPostScriptView *ps_view = control->priv->ps_view;
 
-	/*ugh. the possibilities! */
+	/* ugh. the possibilities! */
 	switch (key) {
 	case GDK_space:
 		switch (orientation) {
@@ -356,16 +431,24 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 		}
 		break;
 	case GDK_Left:
-		gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_LEFT, FALSE);
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_LEFT, TRUE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
 		break;
 	case GDK_Right:
-		gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_RIGHT, FALSE);
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_RIGHT, TRUE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 		break;
 	case GDK_Up:
-		gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_UP, FALSE);
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_UP, TRUE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
 		break;
 	case GDK_Down:
-		gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_DOWN, FALSE);
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_DOWN, TRUE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 		break;
 	case GDK_Page_Up:
 		ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
@@ -502,6 +585,33 @@ set_zoom_items_sensitivity(GgvControl *control, gboolean sens)
 		_set_zoom_items_sensitivity(control, control->priv->popup_uic, sens);
 }
 
+static void
+set_auto_fit_items_sensitivity(GgvControl *control, gboolean sens)
+{
+	gchar *prop_val = sens?"1":"0";
+	Bonobo_UIContainer container;
+
+	container = bonobo_ui_component_get_container (control->priv->uic);
+	if (container == CORBA_OBJECT_NIL)
+		return;
+
+	bonobo_ui_component_freeze(control->priv->uic, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitNone",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitWidth",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitPage",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_thaw(control->priv->uic, NULL);	
+}
+
+static void
+set_command_items_sensitivity(GgvControl *control, gboolean sens)
+{
+	set_auto_fit_items_sensitivity(control, sens);
+	set_zoom_items_sensitivity(control, sens);
+}
+
 static BonoboUIVerb ggv_control_verbs[] = {
 		BONOBO_UI_VERB ("ZoomIn",         verb_ZoomIn_cb),
 		BONOBO_UI_VERB ("ZoomOut",        verb_ZoomOut_cb),
@@ -528,7 +638,7 @@ static void
 ggv_control_create_ui (GgvControl *control)
 {
 	GgvControlClass *klass;
-	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf, *pixbuf_small;
 	gchar *pixbuf_xml;
 	int i;
 
@@ -537,32 +647,56 @@ ggv_control_create_ui (GgvControl *control)
 
 	klass = GGV_CONTROL_CLASS(G_OBJECT_GET_CLASS(control));
 
-	bonobo_ui_util_set_ui(control->priv->uic, NULL, "ggv-control-ui.xml",
+	bonobo_ui_component_freeze(control->priv->uic, NULL);
+	bonobo_ui_util_set_ui(control->priv->uic, DATADIR, "ggv-control-ui.xml",
 						  "GGV", NULL);
+								 
+	control->priv->popup_uic = bonobo_control_get_popup_ui_component(BONOBO_CONTROL(control));
+	bonobo_ui_component_freeze(control->priv->popup_uic, NULL);
+	bonobo_ui_util_set_ui(control->priv->popup_uic, NULL, "ggv-postscript-view-ui.xml", "GGV", NULL);
+	bonobo_ui_util_set_ui(control->priv->popup_uic, DATADIR, "ggv-control-ui.xml", "GGV", NULL);
+	
 	pixbuf = gdk_pixbuf_new_from_file(GNOMEICONDIR "/ggv/fitwidth.png", NULL);
 	if(pixbuf != NULL) {
 		pixbuf_xml = bonobo_ui_util_pixbuf_to_xml(pixbuf);
 		bonobo_ui_component_set_prop(control->priv->uic,
-									 "/commands/ZoomToFitWidth",
+									 "/Toolbar/GgvItems/GgvZoomItems/ZoomToFitWidth",
+									 "pixname", pixbuf_xml,
+									 NULL);
+		g_free(pixbuf_xml);
+		pixbuf_small = gdk_pixbuf_scale_simple(pixbuf, 18, 18, GDK_INTERP_TILES);
+		pixbuf_xml = bonobo_ui_util_pixbuf_to_xml(pixbuf_small);
+		bonobo_ui_component_set_prop(control->priv->uic,
+									 "/menu/View/Zoom Items Placeholder/Zoom/ZoomToFitWidth",
+									 "pixname", pixbuf_xml,
+									 NULL);
+		bonobo_ui_component_set_prop(control->priv->popup_uic,
+									 "/popups/button3/ZoomMenu/ZoomToFitWidth",
 									 "pixname", pixbuf_xml,
 									 NULL);
 		g_free(pixbuf_xml);
 		gdk_pixbuf_unref(pixbuf);
+		gdk_pixbuf_unref(pixbuf_small);
 	}
-								 
-	control->priv->popup_uic = bonobo_control_get_popup_ui_component(BONOBO_CONTROL(control));
-	bonobo_ui_util_set_ui(control->priv->popup_uic, NULL, "ggv-postscript-view-ui.xml", "GGV", NULL);
-	bonobo_ui_util_set_ui(control->priv->popup_uic, NULL, "ggv-control-ui.xml", "GGV", NULL);
-	
+
 	for(i = 0; zoom_level_items[i].path; i++) {
 		bonobo_ui_component_add_listener(control->priv->uic,
 										 zoom_level_items[i].path + strlen("/commands/"),
 										 listener_ZoomLevel_cb, control);
 	}
+	bonobo_ui_component_add_listener(control->priv->uic,
+									 "AutoFitNone",
+									 listener_AutoFitMode_cb, control);
+	bonobo_ui_component_add_listener(control->priv->uic,
+									 "AutoFitWidth",
+									 listener_AutoFitMode_cb, control);
+	bonobo_ui_component_add_listener(control->priv->uic,
+									 "AutoFitPage",
+									 listener_AutoFitMode_cb, control);
 
-	set_zoom_items_sensitivity(control, ggv_postscript_view_get_page_count(control->priv->ps_view) > 0);
+	set_command_items_sensitivity(control, ggv_postscript_view_get_page_count(control->priv->ps_view) > 0);
 
-	sync_zoom_level_items(control);
+	sync_command_state(control);
 
 	bonobo_ui_component_add_verb_list_with_data(control->priv->uic,
 												ggv_control_verbs,
@@ -603,6 +737,8 @@ ggv_control_create_ui (GgvControl *control)
 		control->priv->zoom_control = zoom_control;
 		control->priv->zoom_adj = zoom_adj;
 	}
+	bonobo_ui_component_thaw(control->priv->uic, NULL);
+	bonobo_ui_component_thaw(control->priv->popup_uic, NULL);
 }
 
 static void
@@ -696,7 +832,7 @@ ps_view_property_changed_handler(BonoboListener    *listener,
         GgvControl *control = GGV_CONTROL(data);
 
         if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:page")) {
-			set_zoom_items_sensitivity(control, BONOBO_ARG_GET_INT(any) != -1);
+			set_command_items_sensitivity(control, BONOBO_ARG_GET_INT(any) != -1);
         }
 }
 
