@@ -577,7 +577,7 @@ verb_HelpAbout(BonoboUIComponent *uic, gpointer data, const char *cname)
 		about = gnome_about_new (
 			_("Gnome Ghostview"),
 			VERSION,
-			_("Copyright (C) 1998-2002 The Free Software Foundation"),
+			"Copyright \xc2\xa9 1998-2002 Free Software Foundation, Inc.",
 			_("The GNOME PostScript document previewer"),
 			authors,
 			documenters,
@@ -1021,7 +1021,7 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
 {
         CORBA_Environment ev;
         Bonobo_PersistFile pf;
-        gchar *title, *unescaped;
+        gchar *title, *unescaped, *utf8_title;
 
         g_return_val_if_fail(win != NULL, FALSE);
         g_return_val_if_fail(GGV_IS_WINDOW(win), FALSE);
@@ -1045,8 +1045,10 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
         unescaped = gnome_vfs_unescape_string_for_display(win->filename);
         title = g_strconcat(_("GGV: "), unescaped, NULL);
         g_free(unescaped);
-        gtk_window_set_title(GTK_WINDOW(win), title);
+        utf8_title = g_locale_to_utf8(title, -1, NULL, NULL, NULL);
         g_free(title);
+        gtk_window_set_title(GTK_WINDOW(win), utf8_title);
+        g_free(utf8_title);
 
         if(BONOBO_EX(&ev)) {
                 CORBA_exception_free(&ev);
